//
//  QLController.mm
//
//  Created by  on 2007-12-07.
//  Copyright (c) 2007 . All rights reserved.
//

#import "QLController.h"
#import "FileSystemItem.h"
#import "QuickLook.h"

#define QLPreviewPanel NSClassFromString(@"QLPreviewPanel")

@interface QLController (Private)
- (void)quickLookSelectedItems;
- (FileSystemItem*)visibleItemWithURL:(NSURL*)URL;
@end

@implementation QLController
- (void)awakeFromNib
{
	// First, load the Quick Look framework and set the delegate
	quickLookAvailable = [[NSBundle bundleWithPath:@"/System/Library/PrivateFrameworks/QuickLookUI.framework"] load];
	if(quickLookAvailable)
		[[[QLPreviewPanel sharedPreviewPanel] windowController] setDelegate:self];
}


// This is the delegate method
// It should return the frame for the item represented by the URL
// If an empty frame is returned then the panel will fade in/out instead
- (NSRect)previewPanel:(NSPanel*)panel frameForURL:(NSURL*)URL
{
	NSRect frame = NSMakeRect(0, 0, 0, 0);
	FileSystemItem* item = [self visibleItemWithURL:URL];
	
	if(item)
	{
		frame           = [outlineView rectOfRow:[outlineView rowForItem:item]];
		frame.origin    = [outlineView convertPoint:frame.origin toView:nil];
		frame.origin    = [[outlineView window] convertBaseToScreen:frame.origin];
		frame.origin.y -= frame.size.height;
	}

	return frame;
}

- (BOOL)userDidPressSpaceInOutlineView:(KeyForwardingOutlineView*)anOutlineView
{
	if(quickLookAvailable)
	{
		// If the user presses space when the preview panel is open then we close it
		if([[QLPreviewPanel sharedPreviewPanel] isOpen])
			[[QLPreviewPanel sharedPreviewPanel] closeWithEffect:2];
		else
		{
			// Otherwise, set the current items
			[self quickLookSelectedItems];
			// And then display the panel
			[[QLPreviewPanel sharedPreviewPanel] makeKeyAndOrderFrontWithEffect:2];
			// Restore the focus to our window to demo the selection changing, scrolling (left/right)
			// and closing (space) functionality
			[[outlineView window] makeKeyWindow];
		}
	}
	return YES;
}

- (BOOL)userDidPressRightInOutlineView:(KeyForwardingOutlineView*)outlineView
{
	BOOL keyWasHandled = NO;

	if(quickLookAvailable && [[QLPreviewPanel sharedPreviewPanel] isOpen])
	{
		[[QLPreviewPanel sharedPreviewPanel] selectNextItem];
		keyWasHandled = YES;
	}

	return keyWasHandled;
}

- (BOOL)userDidPressLeftInOutlineView:(KeyForwardingOutlineView*)outlineView
{
	BOOL keyWasHandled = NO;

	if(quickLookAvailable && [[QLPreviewPanel sharedPreviewPanel] isOpen])
	{
		[[QLPreviewPanel sharedPreviewPanel] selectPreviousItem];
		keyWasHandled = YES;
	}

	return keyWasHandled;
}

- (void)quickLookSelectedItems
{
	if(quickLookAvailable)
	{
		NSArray* items         = [outlineView selectedItems];
		NSMutableArray* URLs   = [NSMutableArray arrayWithCapacity:[items count]];
		unsigned int itemCount = [items count];

		for(unsigned int index = 0; index < itemCount; index += 1)
		{
			FileSystemItem* item = [items objectAtIndex:index];
			[URLs addObject:[NSURL fileURLWithPath:[item fullPath]]];
		}

		// The code above just gathers an array of NSURLs representing the selected items,
		// to set here
		[[QLPreviewPanel sharedPreviewPanel] setURLs:URLs currentIndex:0 preservingDisplayState:YES];
	}
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	if(quickLookAvailable)
	{
		// If the user changes the selection while the panel is open then update the current items
		if([[QLPreviewPanel sharedPreviewPanel] isOpen])
			[self quickLookSelectedItems];
	}
}

- (FileSystemItem*)visibleItemWithURL:(NSURL*)URL
{
	NSString* fullPath   = [URL path];
	FileSystemItem* item = nil;
	NSRange visibleRows = [outlineView rowsInRect:[outlineView bounds]];
	int row, endRow;

	for(row = visibleRows.location, endRow = row + visibleRows.length; row <= endRow; ++row)
	{
		FileSystemItem* rowItem = [outlineView itemAtRow:row];
		if([[rowItem fullPath] isEqualToString:fullPath])
		{
			item = [outlineView itemAtRow:row];
			break;
		}
	}

	return item;
}

// Data source stuff
// All taken directly from sample code in the NSOutlineView docs, “Writing an Outline View Data Source” 
- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    return (item == nil) ? 1 : [item numberOfChildren];
}
 
- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
    return (item == nil) ? YES : ([item numberOfChildren] != -1);
}
 
- (id)outlineView:(NSOutlineView *)outlineView
    child:(int)index
    ofItem:(id)item
{
    return (item == nil) ? [FileSystemItem rootItem] : [item childAtIndex:index];
}
 
- (id)outlineView:(NSOutlineView *)outlineView
    objectValueForTableColumn:(NSTableColumn *)tableColumn
    byItem:(id)item
{
    return (item == nil) ? @"/" : [item relativePath];
}
@end
